#!/bin/bash
# Copyright (c) 1992-2018 ESET, spol. s r.o. All Rights Reserved
# set -xv

#- N-ABLE RMM Specific

rmm="solarwinds"

function displayHelp {
    echo ""
    echo "Syntax: epSolarwinds.sh {INSTALL|UNINSTALL|ACTIVATE|DEACTIVATE|CONFIGURE|UPDATE|UPGRADE|SCAN} [OPTIONS]"
    echo ""
    echo "Note: this command needs admin privileges to run"
    echo "Note: the user needs to accept the End User License Agreement by including the parameter --accept_eula"
    echo ""
    echo "   INSTALL: install and activate ESET Endpoint Protection products"
    echo "            OPTIONS: --product {EEA|EES}"
    echo "                    [--activation {activationKey|ELA_username ELA_password ELA_publicID}]"
    echo "            EXAMPLES: sudo ./epSolarwinds.sh Install EEA --accept_eula"
    echo "                      sudo ./epSolarwinds.sh Install EES --activation \"xxxx-xxxx-xxxx-xxxx\" --accept_eula"
    echo "                      sudo ./epSolarwinds.sh Install EES --activation \"me@acme.com Password1 ABC-123-DEF\" --accept_eula"
    echo ""
    echo "   UNINSTALL: uninstalls current ESET product"
    echo "            EXAMPLES: sudo ./epSolarwinds.sh Uninstall  --accept_eula"
    echo ""
    echo "   ACTIVATE: activate ESET Endpoint Protection products"
    echo "            OPTIONS: --activation {activationKey|ELA_username ELA_password ELA_publicID}"
    echo "            EXAMPLES: sudo ./epSolarwinds.sh Activate --activation \"xxxx-xxxx-xxxx-xxxx\" --accept_eula"
    echo "                      sudo ./epSolarwinds.sh Activate --activation \"me@acme.com Password1 ABC-123-DEF\" --accept_eula"
    echo ""
    echo "   DEACTIVATE: deactivate ESET Endpoint Protection products"
    echo "            EXAMPLES: sudo ./epSolarwinds.sh Deactivate --accept_eula"
    echo ""
    echo "   CONFIGURE: apply a configuration file, using a local file or the URL to a network file"
    echo "            OPTIONS: {--file file|--url url}"
    echo "            EXAMPLES: sudo ./epSolarwinds.sh CONF --file 0000.xml --accept_eula"
    echo "                      sudo ./epSolarwinds.sh CONF --url \"https://www.acme.com/0000.xml\" --accept_eula"
    echo ""
    echo "   UPDATE: update AV Modules"
    echo "            EXAMPLES: sudo ./epSolarwinds.sh Update --accept_eula"
    echo ""
    echo "   UPGRADE: upgrade ESET product to another and/or a new version"
    echo "            OPTIONS: --product {EEA|EES}" 
    echo "            EXAMPLES: sudo ./epSolarwinds.sh Upgrade --product EES --accept_eula"
    echo ""
    echo "   SCAN: perform on-demand scan"
    echo "            OPTIONS: --targets {file|folder|pipeSeparatedMacros}" 
    echo "                     --profile profile"
    echo "            DEFAULT: --targets '\${DriveAll}|\${DriveAllBoot}|\${Memory}' "
    echo "                     --profile '\@Smart Scan'"
    echo "            EXAMPLES: sudo ./epSolarwinds.sh scan --accept_eula"
    echo "                      sudo ./epSolarwinds.sh scan --accept_eula --targets \"/tmp\""
    echo "                      sudo ./epSolarwinds.sh scan --accept_eula --profile \"@profile\""
    echo ""
}

TASK=$1
shift

echo task = $TASK
echo args = $@
while [ $# -ne 0 ]; do
    case $1 in
        "--product")
            echo "product parameter found: $2"
            PRODUCT=$(echo "$2" | tr '[a-z]' '[A-Z]')
            shift 2
            ;;
        "--password")
            echo "password parameter found: $2"
            PASSWORD="$2"
            shift 2
            ;;
         "--activation")
            echo "activation parameter found: $2"
            LICENSE="$2" 
            shift 2
            ;;
         "--targets")
            echo "target parameter found: $2"
            SCAN_TARGET="$2"
            shift 2
            ;;
        "--profile")
            echo "profile parameter found: $2"
            SCAN_PROFILE="$2"
            shift 2
            ;;
        "--monitor")
            echo "monitor parameter found: $2"
            MONITOR=$(echo "$2" | tr '[A-Z]' '[a-z]')
            shift 2
            ;;
        "--language")
            echo "language parameter found: $2"
            LANGUAGE="$2"
            shift 2
            ;;
        "--file")
            echo "file parameter found $2"
            FILE="$2"
            shift 2
            ;;
        "--url")
            echo "url parameter found $2"
            rm -rf "/tmp/config.xml"
            curl "$2" -o "/tmp/config.xml" -s
            FILE="/tmp/config.xml"
            shift 2
            ;;
        "--accept_eula")
            echo "accept_eula parameter found"
            ACCEPT="ACCEPT"
            shift 1
            ;;
        "--debug")
            echo "debug parameter found"
            DEBUG="--debug"
            shift 1
            ;;
        *)
            echo "unexpected parameter found: $1"
            displayHelp
            exit
            ;;
    esac
done

version=1.1.1904.142


scriptVersion=$version

# #---------------------------------------- Setup variables

#hash URL
hashUrl="https://update.esetusa.com/plugin/ermm/mac/ermmi.php"
#download URL
downloadUrl="https://update.esetusa.com/plugin/ermm/mac/ermmi"
echo downloadUrl = $downloadUrl
#plugins folder
plugins="/Library/Application Support/ESET/$rmm-plugin.$scriptVersion/ermmi"
#logs folder
logs="/var/log/esets"
if [ ! -d $logs ]; then
    mkdir $logs
fi
#ermmi executable path
ermmi="$plugins/ermmi"
#settings file
plist="$plugins/ermmi.plist"
#log file path
#log="$logs/ermmi.log"

dd=`date "+%Y%m%d-%H%M%S"`
outletFolder="$logs/$rmm-v$scriptVersion-$dd-"

_rmm="--rmm=$rmm"
_deploy="--deploy=EEA"
if [ ! -z "$PRODUCT" ]; then
    _deploy="--deploy=$PRODUCT"
fi
_uninstall="--deploy=uninstall"
_activation=""
if [ ! -z "$LICENSE" ]; then
    _activation="--license=\"$LICENSE\""
fi
_language="--language=en"
if [ ! -z "$LANGUAGE" ]; then
    _target="--language=\"$LANGUAGE\""
fi
_file=""
if [ ! -z "$FILE" ]; then
    _file="--file=\"$FILE\""
fi
_target="--file=\"/\""
if [ ! -z "$SCAN_TARGET" ]; then
    _target="--file=\"$SCAN_TARGET\""
fi
_profile="--scan-profile=\"@Smart scan\""
if [ ! -z "$SCAN_PROFILE" ]; then
    _profile="--scan-profile=\"$SCAN_PROFILE\""
fi
_monitor="-m=\"$MONITOR\""
_debug="$DEBUG"

#---------------------------------------------- Functions

function download {
	curl "$downloadUrl" -o "$ermmi" -s
	chmod +x "$ermmi"
	defaults write "$plist" LastSuccessfulCheck $(date +%s) 2>&1
}

function check_if_root {
    if [ "$EUID" -ne 0 ]; then 
        echo "Please run as root or use sudo"
        displayHelp
        exit
    fi
}

function check_eula {
    if [ "$ACCEPT" == "ACCEPT" ]; then
    	_acceptEULA="--accept-eula"
    else
	    echo "Must Accept End-User License Agreement First"
        displayHelp
	    exit
    fi
}

function create_ERMMI_environment {
    if [ ! -f "$ermmi" ]; then
        # echo $ermmi not found, downloading...
        # if [ ! -d "$logs" ]; then
        #     mkdir -p "$logs"
        # fi	

        if [ ! -d "$plugins" ]; then
            mkdir -p "$plugins"
        fi
        download
    else
        #ermmi does exist, let's see if it needs updated
        #this checks if the plist value exists
        if lastcheck=$(( defaults read "$plist" LastSuccessfulCheck ) 2>&1); then
            #get the date 1 day ago
            yesterday=$(date -v-1d +%s)
            # echo lastcheck,yesterday = "$lastcheck,$yesterday"

            #if the last update check is older than 1 day, update
            if [ "$lastcheck" -lt "$yesterday" ]; then
                #check md5 hash
                localHash=($(md5 -q "$ermmi"))
                remoteHash=$(curl -s $hashUrl)
                if [ "$localHash" != "$remoteHash" ]; then
                    # echo $ermmi outdated, downloading...
                    #hashes don't match, time to update
                    download
                fi
            fi
        else
            # echo $ermmi ... downloading
            download
        fi
    fi
}

function doTask {
    title=$1
    outlet="$outletFolder$title.log"
    echo "logs to $outlet"
    echo "command: $2"
    echo "$(date +%F_%T%Z)" >> "$outlet"
    echo "$2" >> "$outlet"
    eval $2 >> "$outlet" 2>&1
}

#- MAIN --------------------------------------

check_if_root
# echo "Running as root, as it should be"

check_eula
# echo "EULA accepted"

create_ERMMI_environment
# echo "ermmi environment created"

case $TASK in
    MO*)
        doTask Monitor "\"$ermmi\" $_acceptEULA $_rmm $_debug $_monitor"
        ;;
    IN*)
        doTask Install "\"$ermmi\" $_acceptEULA $_rmm $_debug $_deploy $_activation $_language"
        ;;
    UN*)
        doTask Uninstall "\"$ermmi\" $_acceptEULA $_rmm $_debug $_uninstall" 
        ;;
    AC*)
        doTask Activate "\"$ermmi\" $_acceptEULA $_rmm $_debug --task=\"activate\" $_activation" 
        ;;
    DE*)
        doTask Deactivate "\"$ermmi\" $_acceptEULA $_rmm $_debug --task=\"deactivate\""
        ;;
    CO*)
        doTask Configure  "\"$ermmi\" $_acceptEULA $_rmm $_debug --task=\"import-settings\" $_file"
        ;;
    UPD*)
        doTask Update "\"$ermmi\" $_acceptEULA $_rmm $_debug --task=\"update\"" 
        ;;
    UPG*)
        doTask Upgrade "\"$ermmi\" $_acceptEULA $_rmm $_debug $_deploy $_activation"
        ;;
    SC*)
        doTask Scan "\"$ermmi\" $_acceptEULA $_rmm $_debug --task=\"scan\" $_target $_profile"
        ;;
    *)
        displayHelp
        ;;
esac